"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FishLspCodeActionTriggerKind = exports.CodeActionKind = void 0;
exports.getTriggerKind = getTriggerKind;
const LSP = __importStar(require("vscode-languageserver"));
class CodeActionKind {
    value;
    static sep = '.';
    static Empty = new CodeActionKind(LSP.CodeActionKind.Empty);
    static Refactor = new CodeActionKind(LSP.CodeActionKind.Refactor);
    static RefactorToFunction = CodeActionKind.Refactor.append('function');
    static RefactorToVariable = CodeActionKind.Refactor.append('variable');
    static QuickFix = new CodeActionKind(LSP.CodeActionKind.QuickFix);
    static QuickFixFunctionName = CodeActionKind.QuickFix.append('functionName');
    static QuickFixExtraEnd = CodeActionKind.QuickFix.append('extraEnd');
    static QuickFixMissingEnd = CodeActionKind.QuickFix.append('missingEnd');
    static Source = new CodeActionKind(LSP.CodeActionKind.Source);
    static SourceRemoveUnused = CodeActionKind.Source.append('removeUnused');
    static SourceRemoveUnreachable = CodeActionKind.Source.append('removeUnreachable');
    static SourceFixAll = new CodeActionKind(LSP.CodeActionKind.SourceFixAll);
    constructor(value) {
        this.value = value;
    }
    equals(other) {
        return this.value === other.value;
    }
    /**
       * Checks if `other` is a sub-kind of this `CodeActionKind`.
       *
       * The kind `"refactor.extract"` for example contains `"refactor.extract"` and ``"refactor.extract.function"`,
       * but not `"unicorn.refactor.extract"`, or `"refactor.extractAll"` or `refactor`.
       *
       * @param other Kind to check.
       */
    contains(other) {
        return this.equals(other) || this.value === '' || other.value.startsWith(this.value + CodeActionKind.sep);
    }
    /**
       * Checks if this code action kind intersects `other`.
       *
       * The kind `"refactor.extract"` for example intersects `refactor`, `"refactor.extract"` and ``"refactor.extract.function"`,
       * but not `"unicorn.refactor.extract"`, or `"refactor.extractAll"`.
       *
       * @param other Kind to check.
       */
    intersects(other) {
        return this.contains(other) || other.contains(this);
    }
    /**
       * Create a new kind by appending a more specific selector to the current kind.
       *
       * Does not modify the current kind.
       */
    append(part) {
        return new CodeActionKind(this.value + CodeActionKind.sep + part);
    }
}
exports.CodeActionKind = CodeActionKind;
var FishLspCodeActionTriggerKind;
(function (FishLspCodeActionTriggerKind) {
    FishLspCodeActionTriggerKind[FishLspCodeActionTriggerKind["ALL"] = 0] = "ALL";
    FishLspCodeActionTriggerKind[FishLspCodeActionTriggerKind["REFACTOR"] = 1] = "REFACTOR";
    FishLspCodeActionTriggerKind[FishLspCodeActionTriggerKind["QUICKFIX"] = 2] = "QUICKFIX";
})(FishLspCodeActionTriggerKind || (exports.FishLspCodeActionTriggerKind = FishLspCodeActionTriggerKind = {}));
function getTriggerKind(document, params) {
    const range = params.range;
    if (range.start.line === 0 && range.start.character === 0
        && range.end.line === document.lineCount - 1 && range.end.character === 0) {
        return FishLspCodeActionTriggerKind.ALL;
    }
    else {
        return FishLspCodeActionTriggerKind.REFACTOR;
    }
}
