"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorCodes = void 0;
const vscode_languageserver_1 = require("vscode-languageserver");
var ErrorCodes;
(function (ErrorCodes) {
    ErrorCodes.missingEnd = 1001;
    ErrorCodes.extraEnd = 1002;
    ErrorCodes.zeroIndexedArray = 1003;
    ErrorCodes.sourceFileDoesNotExist = 1004;
    ErrorCodes.singleQuoteVariableExpansion = 2001;
    ErrorCodes.usedAlias = 2002;
    ErrorCodes.usedUnviersalDefinition = 2003;
    ErrorCodes.testCommandMissingStringCharacters = 3001;
    ErrorCodes.missingQuietOption = 3002;
    ErrorCodes.expansionInDefinition = 3003;
    ErrorCodes.codes = {
        [ErrorCodes.missingEnd]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Error,
            code: ErrorCodes.missingEnd,
            codeDescription: { href: 'https://fishshell.com/docs/current/cmds/end.html' },
            source: 'fish-lsp',
            message: 'missing closing token',
        },
        [ErrorCodes.extraEnd]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Error,
            code: ErrorCodes.extraEnd,
            codeDescription: { href: 'https://fishshell.com/docs/current/cmds/end.html' },
            source: 'fish-lsp',
            message: 'extra closing token',
        },
        [ErrorCodes.zeroIndexedArray]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Error,
            code: ErrorCodes.zeroIndexedArray,
            codeDescription: { href: 'https://fishshell.com/docs/current/language.html#slices' },
            source: 'fish-lsp',
            message: 'invalid array index',
        },
        [ErrorCodes.sourceFileDoesNotExist]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Error,
            code: ErrorCodes.sourceFileDoesNotExist,
            codeDescription: { href: 'https://fishshell.com/docs/current/cmds/source.html' },
            source: 'fish-lsp',
            message: 'source filename does not exist',
        },
        [ErrorCodes.singleQuoteVariableExpansion]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Warning,
            code: ErrorCodes.singleQuoteVariableExpansion,
            codeDescription: { href: 'https://fishshell.com/docs/current/language.html#variable-expansion' },
            source: 'fish-lsp',
            message: 'non-escaped expansion variable in single quote string',
        },
        [ErrorCodes.usedAlias]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Warning,
            code: ErrorCodes.usedAlias,
            codeDescription: { href: 'https://fishshell.com/docs/current/cmds/alias.html' },
            source: 'fish-lsp',
            message: 'alias used, prefer using functions instead',
        },
        [ErrorCodes.usedUnviersalDefinition]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Warning,
            code: ErrorCodes.usedUnviersalDefinition,
            codeDescription: { href: 'https://fishshell.com/docs/current/language.html#universal-variables' },
            source: 'fish-lsp',
            message: 'Universal scope set in non-interactive session',
        },
        [ErrorCodes.testCommandMissingStringCharacters]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Warning,
            code: ErrorCodes.testCommandMissingStringCharacters,
            codeDescription: { href: 'https://fishshell.com/docs/current/cmds/test.html#examples' },
            source: 'fish-lsp',
            message: 'test command string check, should be wrapped as a string',
        },
        [ErrorCodes.missingQuietOption]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Warning,
            code: ErrorCodes.missingQuietOption,
            codeDescription: { href: 'https://fishshell.com/docs/current/search.html?q=-q' },
            source: 'fish-lsp',
            message: 'Conditional command should include a silence option',
        },
        [ErrorCodes.expansionInDefinition]: {
            severity: vscode_languageserver_1.DiagnosticSeverity.Warning,
            code: ErrorCodes.expansionInDefinition,
            codeDescription: { href: 'https://fishshell.com/docs/current/cmds/set.html' },
            source: 'fish-lsp',
            message: 'Variable definition should not include expansion character',
        },
    };
})(ErrorCodes || (exports.ErrorCodes = ErrorCodes = {}));
