"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDiagnostics = getDiagnostics;
const tree_sitter_1 = require("../utils/tree-sitter");
const node_types_1 = require("./node-types");
const errorCodes_1 = require("./errorCodes");
const file_operations_1 = require("../utils/file-operations");
const cli_1 = require("../cli");
function getDiagnostics(root, doc) {
    let diagnostics = [];
    // compute in single pass
    for (const node of (0, tree_sitter_1.getChildNodes)(root)) {
        if (node.isError) {
            const found = (0, node_types_1.findErrorCause)(node.children);
            const prebuilt = errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.missingEnd];
            if (found) {
                diagnostics.push({
                    range: (0, tree_sitter_1.getRange)(found),
                    ...prebuilt,
                });
            }
        }
        if ((0, node_types_1.isExtraEnd)(node)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.extraEnd],
            });
        }
        if ((0, node_types_1.isZeroIndex)(node)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.zeroIndexedArray],
            });
        }
        if ((0, node_types_1.isSingleQuoteVariableExpansion)(node)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.singleQuoteVariableExpansion],
            });
        }
        if ((0, node_types_1.isAlias)(node)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.usedAlias],
            });
        }
        if ((0, node_types_1.isUniversalDefinition)(node) && !doc.uri.split('/').includes('conf.d')) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.usedUnviersalDefinition],
            });
        }
        if ((0, node_types_1.isSourceFilename)(node) && node.type !== 'subshell' && node.text.includes('/') && !file_operations_1.SyncFileHelper.exists(node.text)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.sourceFileDoesNotExist],
            });
        }
        if ((0, node_types_1.isTestCommandVariableExpansionWithoutString)(node)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.testCommandMissingStringCharacters],
            });
        }
        if ((0, node_types_1.isConditionalWithoutQuietCommand)(node)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.missingQuietOption],
            });
        }
        if ((0, node_types_1.isVariableDefinitionWithExpansionCharacter)(node)) {
            diagnostics.push({
                range: (0, tree_sitter_1.getRange)(node),
                ...errorCodes_1.ErrorCodes.codes[errorCodes_1.ErrorCodes.expansionInDefinition],
            });
        }
    }
    if (cli_1.config.fish_lsp_diagnostic_disable_error_codes.length > 0) {
        for (const errorCode of cli_1.config.fish_lsp_diagnostic_disable_error_codes) {
            diagnostics = diagnostics.filter(diagnostic => diagnostic.code !== errorCode);
        }
    }
    return diagnostics;
}
