"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FishInlayHint = void 0;
exports.inlayHintsProvider = inlayHintsProvider;
const exec_1 = require("./utils/exec");
const node_types_1 = require("./utils/node-types");
const tree_sitter_1 = require("./utils/tree-sitter");
const workspace_symbol_1 = require("./workspace-symbol");
// https://vscode-api.js.org/interfaces/vscode.InlayHintsProvider.html#onDidChangeInlayHints
// https://github.com/youngjuning/vscode-api.js.org/blob/9120b31/vscode.d.ts#L5174
async function inlayHintsProvider(document, range, analyzer) {
    const result = [];
    const nodes = analyzer.getNodes(document);
    const insideRange = (node) => (0, workspace_symbol_1.containsRange)(range, (0, tree_sitter_1.getRange)(node));
    const isPrintableCommand = (node) => node.text.startsWith('printf') || node.text.startsWith('echo'); /* change to printflsp */
    const _isStringCommand = (node) => node.text.startsWith('string');
    const isInlayHint = (node) => {
        if ((0, node_types_1.isPipe)(node)) {
            const first = node.firstNamedChild;
            const second = first?.nextNamedSibling;
            if (!first || !second) {
                return false;
            }
            return ((0, node_types_1.isCommand)(first) &&
                (0, node_types_1.isCommand)(second) &&
                isPrintableCommand(node) &&
                second.firstChild?.text === 'string');
        }
    };
    const hintNodes = nodes
        .filter(insideRange)
        .filter(isInlayHint);
    await Promise.all(hintNodes.map(async (node) => {
        const text = node.text;
        const range = (0, tree_sitter_1.getRange)(node).end;
        // try script escape in shell?
        // set privs to not have write access
        // read lines from file to store variables?
        //const out = await execCmd(`fish -i --command "${text.toString().trim()} | string escape --style=script --no-quoted"`)
        const out = await (0, exec_1.execPrintLsp)(text);
        const value = !out || out.startsWith('Error') ? 'Error' : out;
        //const value = out.join().startsWith('Error') ? `{${out.join()}}`;
        const toolTip = {
            kind: 'markdown',
            value: [
                '```fish',
                text,
                '```',
                '___',
                '```text',
                out,
                '```',
            ].join('\n'),
        };
        const item = FishInlayHint.create(value, range, toolTip);
        result.unshift(item);
    }));
    return result;
}
var FishInlayHint;
(function (FishInlayHint) {
    function create(label, position, toolTip) {
        return {
            label,
            position,
            paddingLeft: true,
            tooltip: toolTip,
        };
    }
    FishInlayHint.create = create;
})(FishInlayHint || (exports.FishInlayHint = FishInlayHint = {}));
//import { DocumentUri, InlayHint, InlayHintKind, Range, } from 'vscode-languageserver';
//import { LspDocuments } from '../document';
//import { ConfigManager } from '../configManager';
//import { uriToPath } from '../utils/translation';
//import { Analyzer } from '../analyze';
//import {getRange, pointToPosition} from '../utils/tree-sitter';
//import * as Locations from '../utils/locations'
//import {isCommandName, isPipe} from '../utils/node-types';
//import {execInlayHintType} from '../utils/exec';
//import { SyntaxNode } from 'web-tree-sitter';
//
//export class FishInlayHintsProvider {
//
//    public static async provideInlayHints(
//        uri: DocumentUri,
//        range: Range,
//        documents: LspDocuments,
//        analyzer: Analyzer,
//        configurationManager: ConfigManager,
//    )   : Promise<InlayHint[]> {
//        const hints: InlayHint[] = [];
//
//        const file = uriToPath(uri);
//        if (!file) return hints;
//
//        const document = documents.get(file);
//        if (!document) return hints;
//
//        const isInlayHintNode = (node: SyntaxNode) => {
//            if (isPipe(node)) {
//                let first = node?.firstNamedChild
//                let second = first?.nextNamedSibling
//                if (!first || !second) return false
//                return (
//                    isCommandName(first) &&
//                    first.text === "printf" &&
//                    isCommandName(second) &&
//                    second.text === "string"
//                );
//            }
//            return false
//        }
//
//        const nodes = analyzer.getNodes(document).filter(isInlayHintNode)
//        console.log(nodes.map(t => t.text))
//        //const config = configurationManager.getInlayHintsEnabled();
//        //for (const node of nodes) {
//        //    let text = ''
//        //    text = await execInlayHintType(`type -t ${node.text} | cut -d ' ' -f1 2>/dev/null`)
//        //    const hint = InlayHint.create({line: node.startPosition.row, character: node.startPosition.column}, text, InlayHintKind.Type)
//        //    hint.paddingLeft = true
//        //    hints.push(hint)
//        //}
//        return hints;
//
//    }
//
//}
