"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocumentSymbolDetail = exports.DocumentationStringBuilder = void 0;
const os_1 = __importDefault(require("os"));
const node_types_1 = require("./node-types");
//import { FishFlagOption, optionTagProvider } from './options';
const translation_1 = require("./translation");
const markdown_builder_1 = require("./markdown-builder");
/**
 * Current CHANGELOG for documentation:
 *     • functions with preceding spaces between their comments keep whitespace between
 *        the comments and the function definition
 *     • @see zoom_out.fish and yarn_reset.fish
 *            -    ~/.config/fish/functions/yarn_reset.fish (shows whole program)
 */
class DocumentationStringBuilder {
    name;
    uri;
    kind;
    inner;
    constructor(name = name, uri = uri, kind = kind, inner = inner) {
        this.name = name;
        this.uri = uri;
        this.kind = kind;
        this.inner = inner;
    }
    get outer() {
        if ((0, node_types_1.isFunctionDefinitionName)(this.inner) || (0, node_types_1.isVariableDefinitionName)(this.inner)) {
            return this.inner.parent;
        }
        return this.inner.previousSibling || null;
    }
    //get tagsString(): string {
    //    return optionTagProvider(this.inner, this.outer)
    //        .map((tag) => {
    //            return tag.toString();
    //        })
    //        .join("\n");
    //}
    /**
     * ~/.config/fish/functions/yarn_reset.fish
     *  causes error, shows entire file instead of just function
     *  meaning that the outer node is being used when it shouldn't be
     */
    get precedingComments() {
        if (this.outer && (0, node_types_1.isProgram)(this.outer)) {
            return getPrecedingCommentString(this.inner);
        }
        if (hasPrecedingFunctionDefinition(this.inner) &&
            (0, node_types_1.isVariableDefinition)(this.inner)) {
            return this.outer?.firstNamedChild?.text + ' ' + this.inner.text;
        }
        return getPrecedingCommentString(this.outer || this.inner);
    }
    get text() {
        const text = this.precedingComments;
        const lines = text.split('\n');
        if (lines.length > 1 && this.outer) {
            const lastLine = this.outer.lastChild?.startPosition.column || 0;
            return lines
                .map((line) => line.replace(' '.repeat(lastLine), ''))
                .join('\n')
                .trimEnd();
        }
        return text;
    }
    get shortenedUri() {
        const uriPath = (0, translation_1.uriToPath)(this.uri);
        return uriPath.replace(os_1.default.homedir(), '~');
    }
    // add this.tagString once further implemented
    toString() {
        //const optionTags = optionTagProvider(this.inner, this.outer);
        //const tagsText = optionTags.map((tag) => tag.toString()).join("\n");
        const symbolString = (0, translation_1.symbolKindToString)(this.kind);
        return new markdown_builder_1.MarkdownBuilder()
            .fromMarkdown([`(${markdown_builder_1.md.italic(symbolString)})`, markdown_builder_1.md.bold(this.name)], `defined in file: ${this.shortenedUri}`, markdown_builder_1.md.separator(), markdown_builder_1.md.codeBlock('fish', this.text))
            .toString();
    }
}
exports.DocumentationStringBuilder = DocumentationStringBuilder;
var DocumentSymbolDetail;
(function (DocumentSymbolDetail) {
    function create(name, uri, kind, inner, _outer = inner.parent || inner.previousSibling || null) {
        return new DocumentationStringBuilder(name, uri, kind, inner).toString();
    }
    DocumentSymbolDetail.create = create;
})(DocumentSymbolDetail || (exports.DocumentSymbolDetail = DocumentSymbolDetail = {}));
function getPrecedingCommentString(node) {
    const comments = [node.text];
    let current = node.previousNamedSibling;
    while (current && current.type === 'comment') {
        comments.unshift(current.text);
        current = current.previousSibling;
    }
    return comments.join('\n');
}
function hasPrecedingFunctionDefinition(node) {
    let current = node.previousSibling;
    while (current) {
        if ((0, node_types_1.isFunctionDefinitionName)(current)) {
            return true;
        }
        current = current.previousSibling;
    }
    return false;
}
